#!/usr/bin/env perl

# ABSTRACT: A command-line program with POD
# PODNAME: command.pl

=head1 SYNOPSIS

    command.pl -h
    command.pl -v

=head1 DESCRIPTION

This is a command whose POD should be available from L<command>.
